import clr
from System import DateTime, Int64, Array
from System.Text import StringBuilder
clr.AddReference("betfair")
from Betfair import *
from lib import BetfairRoundPrice, BetfairRoundPriceDown
import math

def splitOnDelimeter(delimeter, string):
    words = []
    escape = False
    nextWord = StringBuilder()
    for c in string:
        if c == '\\' and not escape:
            escape = True
            continue
        if c == delimeter and not escape:
            words.append(nextWord.ToString())
            nextWord = StringBuilder()
            continue
        escape = False
        nextWord.Append(c)
    words.append(nextWord.ToString())
    return words


def DateTimeFromPosix(milliseconds):
    return DateTime(1970, 1, 1).AddMilliseconds(milliseconds)

	
def nextHighestValidBet(odd1):
    betfairIncrements = [(2, .01), (3, .02), (4, 0.05), (6, .1), (10, .2), (20, .5), (30, 1.0), (50, 2.0), (100, 5.0), (1000, 10.0)]
    
    for odd in betfairIncrements:
        if odd1 < odd[0]:
            return BetfairRoundPrice(odd1 + odd[1])

			
def nextLowestValidBet(odd1):
    betfairIncrements = [(2, .01), (3, .02), (4, 0.05), (6, .1), (10, .2), (20, .5), (30, 1.0), (50, 2.0), (100, 5.0), (1000, 10.0)]
    
    for odd in betfairIncrements:
        if odd1 <= odd[0]:
            return BetfairRoundPrice(odd1 - odd[1])
			
			

class Runner(object):
    def __init__(self, betfairRunner, market):
        self.betfairRunner = betfairRunner
        self.market = market

    @property
    def Id(self):
        return self.betfairRunner.selectionId

    @property
    def Back(self):
        if self.betfairRunner.bestPricesToBack:
            return self.betfairRunner.bestPricesToBack[0].price

    @property
    def BackAvailable(self):
        if self.betfairRunner.bestPricesToBack:
            return self.betfairRunner.bestPricesToBack[0].amountAvailable

    @property
    def Lay(self):
        if self.betfairRunner.bestPricesToLay:
            return self.betfairRunner.bestPricesToLay[0].price

    @property
    def Name(self):
        return self.market.getRunnerName(self.Id)

    @property
    def MarketId(self):
        return self.market.Id

    def __repr__(self):
        # return self.Name + " (Betfair)"
        return self.Name
        
    @property
    def Gateway(self):
        return self.market.Gateway

    def LayAt(self, price, stake):
        response = self.Gateway.Lay(self, price, stake)
        if response.betResults and response.betResults[0].success:
            return response.betResults[0].betId


class Market(object):
    @classmethod
    def fromRecordString(cls, gateway, recordString):
        fields = splitOnDelimeter('~', recordString)
        market = cls(gateway, int(fields[0]))

        market.RunnerCount = int(fields[11])
        market.Name = fields[1]
        market.Path = fields[5]
        market.Status = fields[3]
        market.StartDate = DateTimeFromPosix(int(fields[4]))
        return market

    def __init__(self, gateway, id):
        self.Id = id
        self.Gateway = gateway
        self.__marketResponse = None

    def __repr__(self):
        return "%s (Betfair)" % (self.Name,)

    def GetRunners(self):
        req = GetMarketPricesReq()
        req.marketId = self.Id
        response = self.Gateway.exchangeService.getMarketPrices(req)

        runners = []
        for runnerPrice in response.marketPrices.runnerPrices:
            runners.append(Runner(runnerPrice, self))

        return runners

    def getRunner(self, id):
        return [r for r in self.GetRunners() if r.Id == id][0]

    @property
    def _marketResponse(self):
        if not self.__marketResponse:
            request = GetMarketReq(marketId=self.Id)
            self.__marketResponse = self.Gateway.exchangeService.getMarket(request)
        return self.__marketResponse

    def getRunnerName(self, runnerId):
        for runner in self._marketResponse.market.runners:
            if runner.selectionId == runnerId:
                return runner.name
        return None


class SessionRememberingProxy(object):
    def __init__(self, proxy, sessionToken=None):
        self._realProxy = proxy
        self.sessionToken = sessionToken

    def __getattr__(self, name):
        realAttr = getattr(self._realProxy, name)
        def wrapped(req):
            if not isinstance(req, LoginReq):
                req.header = APIRequestHeader(sessionToken=self.sessionToken)
            resp = realAttr(req)
            self.sessionToken = resp.header.sessionToken
            print resp.errorCode
            return resp
        return wrapped


class BetfairGateway(object):
    def __init__(self, username, password, productId=22):
        self.service = SessionRememberingProxy(BFGlobalService())
        def login():
            request = LoginReq(username=username, password=password, productId=productId)
            response = self.service.login(request)
            assert response.errorCode == LoginErrorEnum.OK, "login failed"
        login()
        self.exchangeService = SessionRememberingProxy(BFExchangeService(), sessionToken=self.service.sessionToken)
        self.MinStake = 2.0
        self.availableFunds = None


    def Back(self, runner, price, amount):
        req = PlaceBetsReq(header=APIRequestHeader())
        bet = PlaceBets(selectionId=runner.Id, price=BetfairRoundPrice(price), size=round(amount,2), marketId=runner.market.Id, bspLiability=0.0)
        req.bets = (bet,)
        response = self.exchangeService.placeBets(req)
        return response


    def GetMarketPL(self, marketID):
        req = GetMarketProfitAndLossReq(header=APIRequestHeader(), marketID = marketID, includeBspBets=False)
        response = self.exchangeService.getMarketProfitAndLoss(req)
        return response


    def GetCurrentBetsLite(self):
        req = GetCurrentBetsLiteReq()
        response = self.exchangeService.getCurrentBetsLite(req)
        return response


    def GetBetMatchesLite(self, betID):
        req = GetBetMatchesLiteReq(betId=Int64(betID))
        response = self.exchangeService.getBetMatchesLite(req)
        return response


        
    def Lay(self, runner, price, amount):
        req = PlaceBetsReq()
        bet = PlaceBets(selectionId=runner.Id, price=BetfairRoundPrice(price),
            size=round(amount,2), marketId=runner.market.Id,
            bspLiability=0.0, betType=BetTypeEnum.L)
        req.bets = (bet,)
        response = self.exchangeService.placeBets(req)
        return response


    def CancelBet(self, betid):
        req = CancelBetsReq()
        req.bets = (CancelBets(betId=Int64(betid)),)
        response = self.exchangeService.cancelBets(req)
        return response
        
    def CancelBetsByMarket(self, marketID):
        req = CancelBetsByMarketReq()
        req.markets = (Int64(marketID),)
        # req.markets = Array[Int64]((100786475))
        response = self.exchangeService.cancelBetsByMarket(req)
        return response   


    def UnmatchedStake(self, betid):
        req = GetBetLiteReq(betId=Int64(betid))
        response = self.exchangeService.getBetLite(req)
        return response.betLite.remainingSize

    def GetMarketName(self, marketid):
        req = GetMarketReq(marketId = int(marketid))
        response = self.exchangeService.getMarket(req)
        return response.market.name
        
    def RoundPrice(self, price):
        return BetfairRoundPrice(price)

    def RoundPriceDown(self, price):
        return BetfairRoundPriceDown(price)


    def GetMarkets(self):
        request = GetAllMarketsReq()
        response = self.exchangeService.getAllMarkets(request)
        for marketDataRecord in splitOnDelimeter(':', response.marketData)[1:]:
            yield Market.fromRecordString(self, marketDataRecord)

    @property
    def AvailableFunds(self):
        if self.availableFunds is None:
            req = GetAccountFundsReq()
            response = self.exchangeService.getAccountFunds(req)
            self.availableFunds = response.availBalance
        return self.availableFunds

    def GetMarketById(self, id):
        return Market(self, id)

